import * as server from "@minecraft/server";
import {
    ARROW_EFFECTS,
    damageItem,
    shootProjectile,
    itemReplace,
    badEffects
} from "./sagCrossbowFunctions";
const world = server.world;
// Определение значений силы для различных материалов
const powerValues = { 'big': 3, };
// Массив псевдослучайных значений для звукового питча
const fakeRNG = [0.991, 1.149, 0.86, 0.851, 0.878, 1.055, 0.996, 0.99, 1.002, 0.887, 0.874, 1.09, 1.1, 0.801, 1.05];
let fakeVAR = 0;

world.beforeEvents.worldInitialize.subscribe((initEvent) => {
    // Регистрация пользовательского компонента арбалета
    initEvent.itemComponentRegistry.registerCustomComponent(
        "sag:crossbow_component",
        {
            // Обработка завершения использования арбалета
            onCompleteUse(data) {
                const item = data.itemStack;
                const player = data.source;
                let potionEffect = "";
                let potionDuration = 1;
                let potionAmplifier = 0;
                // Проверка, является ли выстрел очередью
                const isBurstShot = item.getDynamicProperty('sag:perk') === 'burst_shot' || item.getDynamicProperty('sag:s_perk') === 'burst_shot';
                const clearAmount = isBurstShot ? 3 : 3;

                // Проверка наличия минимум 3 стрел у игрока
                if (player.runCommand(`testfor @s[hasitem={item=minecraft:arrow,quantity=3..}]`).successCount === 0) {
                    // Если меньше 3 стрел, прерываем зарядку
                    return;
                }

                // Проверка наличия стрел у игрока
                if (player.runCommand(`testfor @s[hasitem={item=minecraft:arrow${isBurstShot ? ',quantity=3..' : ''}}]`).successCount > 0) {
                    const isOffhand = player.runCommand('testfor @s[hasitem={item=minecraft:arrow,location=slot.weapon.offhand}]').successCount > 0;

                    // Функция обработки эффекта стрелы
                    const processArrowEffect = (locationFilter) => {
                        for (let i = 6; i < 48; i++) {
                            const command = `testfor @a[hasitem={item=minecraft:arrow,${locationFilter},data=${i}}]`;
                            if (player.runCommand(command).successCount > 0) {
                                player.runCommand(`clear @s arrow ${i} ${clearAmount}`);
                                [potionEffect, potionDuration, potionAmplifier] = ARROW_EFFECTS[i];
                                break;
                            }
                        }
                    };

                    // Обработка стрелы в зависимости от слота
                    isOffhand == true ? processArrowEffect('location=slot.weapon.offhand') : processArrowEffect('quantity=!0');
                    potionEffect == false ? player.runCommand(`clear @s arrow 0 ${clearAmount}`) : player;

                    // Установка динамических свойств арбалета
                    item.setDynamicProperty('sag:target_crossbow', item.typeId);
                    item.setDynamicProperty('sag:p_effect', potionEffect);
                    item.setDynamicProperty('sag:p_duration', potionDuration);
                    item.setDynamicProperty('sag:p_amplifier', potionAmplifier);

                    player.dimension.playSound('crossbow.loading.end', player.location);
                    player.addTag('crossbowHold');

                    // Замена предмета на заряженный арбалет
                    itemReplace(player, item, item.typeId.replace(':crossbow', '_loaded:bow').replace('c1_', '').replace('c2_', '').replace('c3_', ''));
                    server.system.runTimeout(() => {
                        player.removeTag('crossbowHold');
                    }, 7);
                }
            },
            // Обработка использования арбалета
            onUse(data) {
                const item = data.itemStack;
                const typeId = item.typeId;
                if (typeId.includes('_loaded')) {
                    if (!data.source.hasTag('crossbowHold')) {
                        let power = 0;
                        let realPower = 5;
                        let particleIndex = 0;
                        let tag = 'h';
                        let currentArrow;

                        const hasFlame = item.getDynamicProperty('sag:perk') == "flame" || item.getDynamicProperty('sag:s_perk') == "flame";
                        const hasStrider = item.getDynamicProperty('sag:perk') == "strider_arrow" || item.getDynamicProperty('sag:s_perk') == "strider_arrow";

                        // Расчет силы на основе материала
                        for (const material in powerValues) {
                            if (typeId.includes(material)) {
                                power += powerValues[material];
                            }
                        }
                        if (hasFlame) particleIndex = 3;
                        // Учет натяжения арбалета
                        const tension = item.getDynamicProperty('sag:tension');
                        if (tension >= 1) {
                            power += tension;
                        }

                        // Получение чар
                        const enchant = item?.getComponent("minecraft:enchantable");
                        const levelMultishot = enchant?.getEnchantment("multishot")?.level ?? 0;
                        const levelPiercing = enchant?.getEnchantment("piercing")?.level ?? 0;
                        const potionEffect = item.getDynamicProperty('sag:p_effect')
                        const potionDuration = item.getDynamicProperty('sag:p_duration')
                        const potionAmplifier = item.getDynamicProperty('sag:p_amplifier')
                        const perk = item?.getDynamicProperty('sag:perk')
                        const sPerk = item?.getDynamicProperty('sag:s_perk')
                        const hasSwift = item.getDynamicProperty('sag:perk') == "swift_arrow" || item.getDynamicProperty('sag:s_perk') == "swift_arrow";
                        hasSwift ? realPower += 3 : 0;
                        if (potionEffect != "") particleIndex = 5;
                        let doLoop = 3;
                        if (perk)
                            if (perk == "burst_shot") doLoop = 0
                        if (sPerk)
                            if (sPerk == "burst_shot") doLoop = 0
                        // Цикл для обработки очереди выстрелов
                        do {
                            server.system.runTimeout(() => {
                                if (data.source.getComponent('inventory').container.getItem(data.source.selectedSlotIndex) == undefined) {
                                    doLoop = 3
                                    return;
                                }
                                // Обработка мультивыстрела
                                if (enchant && levelMultishot > 0) {
                                    currentArrow = shootProjectile(world, data.source, 'sag:arrow_f', realPower, 0, false, [potionEffect, potionDuration, potionAmplifier], power)
                                    currentArrow.setProperty('sag:particle_index', particleIndex)
                                    currentArrow.addTag(tag)
                                    currentArrow = shootProjectile(world, data.source, 'sag:arrow_f', realPower, 0.1, false, [potionEffect, potionDuration, potionAmplifier], power)
                                    currentArrow.setProperty('sag:particle_index', particleIndex)
                                    currentArrow.addTag(tag)
                                    currentArrow = shootProjectile(world, data.source, 'sag:arrow_f', realPower, -0.1, false, [potionEffect, potionDuration, potionAmplifier], power)
                                    currentArrow.setProperty('sag:particle_index', particleIndex)
                                    currentArrow.addTag(tag)
                                    currentArrow = shootProjectile(world, data.source, 'sag:fake_arrow_f', realPower, 0.2, false, [potionEffect, potionDuration, potionAmplifier], power)
                                    currentArrow.setProperty('sag:particle_index', particleIndex)
                                    currentArrow.addTag('isFromMultishot')
                                    currentArrow.addTag(tag)
                                    currentArrow = shootProjectile(world, data.source, 'sag:fake_arrow_f', realPower, -0.2, false, [potionEffect, potionDuration, potionAmplifier], power)
                                    currentArrow.setProperty('sag:particle_index', particleIndex)
                                    currentArrow.addTag('isFromMultishot')
                                    currentArrow.addTag(tag)
                                    currentArrow = shootProjectile(world, data.source, 'sag:fake_arrow_f', realPower, -0.05, false, [potionEffect, potionDuration, potionAmplifier], power)
                                    currentArrow.setProperty('sag:particle_index', particleIndex)
                                    currentArrow.addTag('isFromMultishot')
                                    currentArrow.addTag(tag)
                                    currentArrow = shootProjectile(world, data.source, 'sag:fake_arrow_f', realPower, 0.05, false, [potionEffect, potionDuration, potionAmplifier], power)
                                    currentArrow.setProperty('sag:particle_index', particleIndex)
                                    currentArrow.addTag('isFromMultishot')
                                    currentArrow.addTag(tag)
                                    currentArrow = shootProjectile(world, data.source, 'sag:fake_arrow_f', realPower, -0.25, false, [potionEffect, potionDuration, potionAmplifier], power)
                                    currentArrow.setProperty('sag:particle_index', particleIndex)
                                    currentArrow.addTag('isFromMultishot')
                                    currentArrow.addTag(tag)
                                    currentArrow = shootProjectile(world, data.source, 'sag:fake_arrow_f', realPower, 0.25, false, [potionEffect, potionDuration, potionAmplifier], power)
                                    currentArrow.setProperty('sag:particle_index', particleIndex)
                                    currentArrow.addTag('isFromMultishot')
                                    currentArrow.addTag(tag)
                                } else if (enchant && levelPiercing > 0) {
                                    // Обработка пробивания
                                    currentArrow = shootProjectile(world, data.source, 'sag:arrow_f', realPower, 0.1, levelPiercing, [potionEffect, potionDuration, potionAmplifier], power)
                                    currentArrow.setProperty('sag:particle_index', particleIndex)
                                    currentArrow.addTag(tag)
                                } else {
                                    // Обычный выстрел
                                    currentArrow = shootProjectile(world, data.source, 'sag:arrow_f', realPower, 0, false, [potionEffect, potionDuration, potionAmplifier], power);
                                    currentArrow.setProperty('sag:particle_index', particleIndex);
                                    currentArrow.addTag(tag)
                                    currentArrow = shootProjectile(world, data.source, 'sag:arrow_f', realPower, 0.1, false, [potionEffect, potionDuration, potionAmplifier], power)
                                    currentArrow.setProperty('sag:particle_index', particleIndex)
                                    currentArrow.addTag(tag)
                                    currentArrow = shootProjectile(world, data.source, 'sag:arrow_f', realPower, -0.1, false, [potionEffect, potionDuration, potionAmplifier], power)
                                    currentArrow.setProperty('sag:particle_index', particleIndex)
                                    currentArrow.addTag(tag)
                                }
                                fakeVAR++;
                                if (fakeVAR >= fakeRNG.length) fakeVAR = 0;
                                // Воспроизведение звука выстрела
                                data.source.playSound('crossbow.shoot', {
                                    location: data.source.location,
                                    pitch: fakeRNG[fakeVAR]
                                })
                                // Нанесение урона предмету
                                damageItem(data.source, doLoop != 3 ? 3 : 1)
                            }, doLoop != 3 ? doLoop * 6 : 0)
                            doLoop++;
                        } while (doLoop < 3)
                        // Замена предмета на незаряженный арбалет
                        itemReplace(data.source, item, item.getDynamicProperty('sag:target_crossbow'));
                    }
                }
                else {
                    // Обработка начала зарядки арбалета
                    const enchant = data.itemStack?.getComponent("minecraft:enchantable");
                    const levelCharge = enchant?.getEnchantment("quick_charge")?.level ?? 0;
                    data.source.dimension.playSound('crossbow.loading.start', data.source.location)
                    server.system.runTimeout(() => {
                        data.source.dimension.playSound('crossbow.loading.middle', data.source.location)
                    }, 2)
                    // Обработка быстрой зарядки
                    if (levelCharge) {
                        if (!data.itemStack.typeId.includes('c' + levelCharge)) {
                            const repTemp = (data.itemStack.typeId).replace('c1_', '').replace('c2_', '').replace('c3_', '');
                            switch (levelCharge) {
                                case 1:
                                    itemReplace(data.source, data.itemStack, 'c' + levelCharge + '_' + repTemp)
                                    break;
                                case 2:
                                    itemReplace(data.source, data.itemStack, 'c' + levelCharge + '_' + repTemp)
                                    break;
                                case 3:
                                    itemReplace(data.source, data.itemStack, 'c' + levelCharge + '_' + repTemp)
                                    break;
                            }
                        }
                    }
                }
            }
        }
    );
});