import * as server from "@minecraft/server";
import "crossbowComponent.js";
// Получаем объект мира для взаимодействия с игроками и сущностями.
const world = server.world;
server.system.runInterval(() => {
    // Перебираем всех игроков в мире.
    for (const player of world.getPlayers()) {
        // Получаем индекс активного слота игрока.
        const cSlot = player.selectedSlotIndex;
        // Получаем предыдущий активный слот из динамического свойства.
        let pSlot = player?.getDynamicProperty('sag:pSlot');
        // Если предыдущий слот существует и отличается от текущего, убираем тег 'crossbowHold'.
        if (pSlot !== undefined && pSlot >= 0) {
            if (pSlot !== cSlot) player.removeTag('crossbowHold');
        }
        // Сохраняем текущий слот как предыдущий.
        player.setDynamicProperty('sag:pSlot', cSlot);
        try {
            // Получаем предмет в активном слоте игрока.
            const currentItem = player.getComponent('inventory').container.getItem(cSlot);
            // Получаем свойства предмета (перки).
            const perk = currentItem?.getDynamicProperty('sag:perk');
            const s_perk = currentItem?.getDynamicProperty('sag:s_perk');
            // Если игрок крадётся и предмет имеет перк "hunting", активируем режим прицеливания.
            if (player.isSneaking && (perk === "hunting" || s_perk === "hunting")) {
                // Если тег прицеливания отсутствует, проигрываем звук начала зарядки арбалета.
                if (!player.hasTag('sag_scoping_c')) {
                    player.playSound('crossbow.loading.start', {
                        pitch: 1.4
                    });
                }
                // Добавляем тег прицеливания и эффект замедления.
                player.addTag('sag_scoping_c');
                player.addEffect('slowness', 10, {
                    amplifier: 8,
                    showParticles: false
                });
            } else {
                // Если игрок не крадётся, убираем тег прицеливания.
                player.removeTag('sag_scoping_c');
            }
        } catch { }
    }
}, 5);

/**
 * Подписываемся на событие попадания снаряда в сущность.
 * Обрабатываем попадание стрелы типа "sag:fake_arrow_2".
 */
world.afterEvents.projectileHitEntity.subscribe(data => {
    // Проверяем, является ли снаряд нужным типом стрелы.
    if (data.projectile.typeId == "sag:arrow_f") {
        // Получаем сущность, в которую попала стрела.
        const entityHit = data.getEntityHit().entity;
        // Получаем урон стрелы из её динамического свойства.
        const power = data.projectile.getDynamicProperty('sag:damage');
        const pr = data.projectile;
        // Получаем владельца стрелы (игрока, который её выпустил).
        const owner = data.projectile.getComponent("minecraft:projectile")?.owner;
        // Получаем счётчик попаданий стрелы.
        const arrow = data.projectile.getDynamicProperty('sag:arrow_counter');
        // Получаем уровень пробивания (piercing).
        let piercing = data.projectile.getDynamicProperty('sag:piercing');
        piercing != undefined ? piercing++ : piercing;
        // Получаем эффект зелья, связанный со стрелой.
        const effect = data.projectile.getDynamicProperty('sag:potion_effect');

        // Увеличиваем счётчик попаданий стрелы.
        data.projectile.setDynamicProperty('sag:arrow_counter', arrow + 1);
        // Наносим урон сущности, учитывая базовый урон и дополнительный.
        entityHit.applyDamage(9 + power, { damagingProjectile: data.projectile, damagingEntity: owner });
        // Если есть эффект зелья, применяем его к сущности.
        if (effect != "") {
            entityHit.addEffect(effect, data.projectile.getDynamicProperty('sag:potion_duration') * 20);
        }
        // Если это первое попадание, удаляем стрелу.
        if (arrow == 0) {
            data.projectile.remove();
            return;
        }
        // Если количество попаданий равно пробиванию, удаляем стрелу.
        if (arrow == piercing) {
            data.projectile.remove();
        }
        // Если пробивание больше 0, добавляем тег 'isFromMultishot'.
        if (piercing != 0) data.projectile.addTag('isFromMultishot');
    } else return;
});
world.afterEvents.entitySpawn.subscribe(data => {
    const entity = data.entity;
    // Проверяем, является ли сущность стрелой.
    if (entity.typeId.includes('arrow')) {
        // Получаем владельца стрелы.
        const owner = entity.getComponent("minecraft:projectile").owner;
        const arrow = entity.getComponent('minecraft:projectile');
        // Получаем предмет в активном слоте владельца.
        const itemStack = owner.getComponent('inventory').container.getItem(owner.selectedSlotIndex);
    } else return;
});

/**
 * Подписываемся на событие получения урона сущностью.
 * Обрабатываем эффекты арбалетов с различными модификаторами.
 */
world.afterEvents.entityHurt.subscribe(data => {
    // Проверяем наличие источника урона и снаряда.
    if (!data?.damageSource || !data.damageSource?.damagingProjectile) return;

    const hurtEntity = data.hurtEntity;
    const damager = data.damageSource.damagingEntity;

    // Проверяем, что атакующий — игрок.
    if (damager.typeId !== "minecraft:player") return;

    let damage = data.damage;
    // Получаем предмет в активном слоте атакующего.
    const itemStack = damager.getComponent('inventory').container.getItem(damager.selectedSlotIndex);
    const tension = itemStack?.getDynamicProperty('sag:tension');

    /**
     * Функция для применения эффектов к сущностям в радиусе.
     * @param {Entity} entity - Целевая сущность.
     * @param {Object} effectType - Параметры эффекта (дальность, тон, громкость).
     * @param {string} particle - Тип частиц.
     * @param {string} sound - Звук, воспроизводимый при активации.
     * @param {number} delay - Задержка перед применением.
     * @param {Function} extraEffect - Дополнительный эффект для каждой сущности.
     */
    const applyEffects = (entity, effectType, particle, sound, delay, extraEffect) => {
        let i = 0;
        // Получаем все сущности в заданном радиусе.
        const targets = damager.dimension.getEntities({
            maxDistance: effectType.maxDistance,
            location: hurtEntity.location
        });

        // Применяем эффекты к каждой сущности.
        targets.forEach(e => {
            if (e.typeId !== 'minecraft:item' && e !== damager && e.typeId !== 'minecraft:xp_orb') {
                i++;
                server.system.runTimeout(() => {
                    e.dimension.spawnParticle(particle, e.location);
                    if (extraEffect) extraEffect(e, i);
                    e.dimension.playSound(sound, e.location, { pitch: effectType.pitch, volume: effectType.volume });
                }, i);
                if (effectType.maxTargets && i >= effectType.maxTargets) return;
            }
        });
    };
});





world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent('sag:egg_grow', {
        onRandomTick: e => {
            const block = e.block;
            if (block.below(1).typeId !== "minecraft:sand") return; // Проверка песка
            const currentStage = block.permutation.getState('sag:egg_hatches');
            const randomChance = Math.random() * 100;
            if (randomChance <= 2.5) { // ~4000 тиков на стадию, 12000 тиков всего
                if (currentStage < 2) {
                    block.setPermutation(block.permutation.withState('sag:egg_hatches', currentStage + 1));
                    block.dimension.spawnParticle('minecraft:crop_growth_emitter', block.center());
                    block.dimension.playSound('block.turtle_egg.crack', block.center());
                } else if (currentStage === 2) {
                    // Замена блока на воздух
                    block.setType('minecraft:air');
                    // Спавн сущности
                    const entity = block.dimension.spawnEntity('sag:brammer_spawn_mob_center', {
                        x: block.location.x + 0.5,
                        y: block.location.y,
                        z: block.location.z + 0.5
                    });
                    // Проигрывание звука
                    block.dimension.playSound('mob.turtle_baby.born', block.center());
                }
            }
        },
    });
});